from datetime import datetime
from typing import Optional

from pydantic import BaseModel, validator


class Subscription(BaseModel):
    flight_id: int
    user_id: int


class SubscriptionRemove(BaseModel):
    flight_id: int


class FlightData(BaseModel):
    id: int
    flight_code: str
    status: Optional[str] = None
    origin: str
    destination: str
    departure_time: Optional[str] = None
    arrival_time: Optional[str] = None
    gate: Optional[str] = None

    @validator("departure_time", "arrival_time", pre=True, always=True)
    def parse_datetime(cls, value):
        if isinstance(value, datetime):
            return value.strftime("%Y-%m-%d %I:%M %p")
        return value
